/******************************************************************/
/* filename :       plot_predlh.g                                 */
/* Description:                                                   */
/*                                                                */
/******************************************************************/

new;
closeall;
library pgraph;
cls;

workpath  = cdir(0);
fileext = "us";

//load path = ^workpath predlhall[]= "predlh_us.txt";
//predlhall       = reshape(predlhall,rows(predlhall)/5,5);
//ti = predlhall[.,1];
//predlhall = predlhall[.,2:5];

// *******************************************************************
// Load Predictive Likelihoods
fname = workpath $+ "\\inputs\\PredictiveLikelihoods.xlsx";

if fileext $== "us";
   // period, gap4, gap3, growth4, growth3, gap4new, growth4new
   // 1986 - 2015
   sheetNum = 1;
   range = "B11:H127";
else;
   // period, gap4, gap3, growth4, growth3, gap4new, growth4new
   // 1983 - 2015
   sheetNum = 2;
   range = "B11:H139"; 
endif;
vls = "";
predlhall = xlsReadM(fname, range, sheetNum, vls);
ti        = predlhall[.,1];
predlhall = predlhall[.,2:7];

// *******************************************************************
// Load States from Gap3
if fileext $== "us";
   fname = workpath $+ "\\inputs\\USJP3vGap\\states_sunspot14_095099_us_gsME07R02.xlsx";
   // Prob s=1
   // 1986 - 2015
   range = "V11:V127";
else;
   fname = workpath $+ "\\inputs\\USJP3vGap\\states_sunspot14_092099_jp_gsME07R02.xlsx";
   // Prob s=1
   // 1983 - 2015
   range = "V11:V139";	
endif;
sheetNum = 1;
vls = "";
Gap3s1 = xlsReadM(fname, range, sheetNum, vls);

// *******************************************************************
// Load States from Gap4
if fileext $== "us";
   fname = workpath $+ "\\inputs\\USJP4vGapC\\states_sunspot132_095099_us_gsME07R02.xlsx";
   // Prob s=1
   // 1986 - 2015
   range = "V11:V127";	
else;
   fname = workpath $+ "\\inputs\\USJP4vGapC\\states_sunspot16_092099_jp_gsME07R02.xlsx";
   // Prob s=1
   // 1983 - 2015
   range = "V11:V139";	
endif;
sheetNum = 1;
vls = "";
Gap4s1 = xlsReadM(fname, range, sheetNum, vls);

// *******************************************************************
// Load States from Growth3
if fileext $== "us";
   fname = workpath $+ "\\inputs\\USJP3vGrowth\\states_sunspot13_095099_us_gsME07R02.xlsx";
   // Prob s=1
   // 1986 - 2015
   range = "U11:U127";
else;
   fname = workpath $+ "\\inputs\\USJP3vGrowth\\states_sunspot14_092099_psifree_jp_gsME07R02.xlsx";
   // Prob s=1
   // 1983 - 2015
   range = "U11:U139";	
endif;
sheetNum = 1;
vls = "";
Growth3s1 = xlsReadM(fname, range, sheetNum, vls);

// *******************************************************************
// Load States from Growth4
if fileext $== "us";
   fname = workpath $+ "\\inputs\\USJP4vGrowthC\\states_sunspot14_095099_psifree_us_gsME07R02.xlsx";
   // Prob s=1
   // 1986 - 2015
   range = "U11:U127";	
else;
   fname = workpath $+ "\\inputs\\USJP4vGrowthC\\states_sunspot13v5_092099_psifree_jp_gsME07R02.xlsx";
   // Prob s=1
   // 1983 - 2015
   range = "U11:U139";	
endif;
sheetNum = 1;
vls = "";
Growth4s1 = xlsReadM(fname, range, sheetNum, vls);

// *******************************************************************
// Load States from Gap4new
if fileext $== "us";
   fname = workpath $+ "\\inputs\\USJP4vGap4G\\states_sunspot14_095099_us_gsME07R02.xlsx";
   // Prob s=1
   // 1986 - 2015
   range = "V11:V127";	
else;
   fname = workpath $+ "\\inputs\\USJP4vGap4G\\states_sunspot14_092099_jp_gsME07R02.xlsx";
   // Prob s=1
   // 1983 - 2015
   range = "V11:V139";	
endif;
sheetNum = 1;
vls = "";
Gap4news1 = xlsReadM(fname, range, sheetNum, vls);

// *******************************************************************
// Load States from Growth4new
if fileext $== "us";
   fname = workpath $+ "\\inputs\\USJPGrowth4G\\states_sunspot15_095099_us_gsME07R02.xlsx";
   // Prob s=1
   // 1986 - 2015
   range = "U11:U127";	
else;
   fname = workpath $+ "\\inputs\\USJPGrowth4G\\states_sunspot14_092099_jp_gsME07R02.xlsx";
   // Prob s=1
   // 1983 - 2015
   range = "U11:U139";	
endif;
sheetNum = 1;
vls = "";
Growth4news1 = xlsReadM(fname, range, sheetNum, vls);

// *******************************************************************
// Combine results

s1all = Gap4s1 ~ Gap3s1 ~ Growth4s1 ~ Growth3s1 ~ Gap4news1 ~ Growth4news1; 

// *******************************************************************

//Declare the structure 
struct plotControl myPlot;

//Initialize the structure 
myplot = plotGetDefaults("xy");
//Turn off the grid 
plotSetGrid(&myPlot, "on");
//Legend
label = "Gap 4"$|"Gap 3"$|"Growth 4"$|"Growth 3"$|"Gap 4 new"$|"Growth 4 new";
location = "top right";
orientation = 0;
plotSetLegend(&myPlot, label, location, orientation);
// Line Style
plotSetLineStyle(&myPlot, (1~3~4~5));
// Line Thickness
plotSetLineThickness(&myPlot, 5);
// color
plotSetLineColor(&myPlot, ("red" $| "blue" $| "green" $| "black" $| "yellow" $| "orange"));
//Set the X-axis label, label font, label font size, and 
plotSetXLabel(&myPlot, "Period", "verdana", 16, "black");
//Set the Y-axis label, label font, label font size, and 
plotSetYLabel(&myPlot, "Pred LogLH", "verdana", 16, "black");

// *******************************************************************

// PLOT 1: 
plotOpenWindow();
plotLayout(2, 2, 1);
//Set the title, title font and title font size 
plotSetTitle(&myPlot, "Cumulative", "verdana", 18);
// Plot the graph
plotXY(myPlot,ti,cumsumc(predlhall));


ti~cumsumc(predlhall);
"Plot 1.1";
"Period versus cumulative Predictive logLH, full sample";
"LogLh for gap4, gap3, growth4, growth3, gap4new, growth4new";
"-----------------------------------------------------------";


plotLayout(2, 2, 2);
//Set the title, title font and title font size 
plotSetTitle(&myPlot, "Cumulative - Post ZLB", "verdana", 18);
// Plot the graph
if fileext $== "us";
   Tfirst = 2009.00;
   Tlast  = 2015.00;
else;
   Tfirst = 1999.00;
   Tlast  = 2015.00;
endif;	
Tfirst_ind    = maxindc(ti .== Tfirst);
Tlast_ind     = maxindc(ti .== Tlast);
plotXY(myPlot,ti[Tfirst_ind:Tlast_ind,.],cumsumc(predlhall[Tfirst_ind:Tlast_ind,.]));

ti[Tfirst_ind:Tlast_ind,.]~cumsumc(predlhall[Tfirst_ind:Tlast_ind,.]);
"Plot 1.2";
"Period versus cumulative Predictive logLH, post ZLB";
"LogLh for gap4, gap3, growth4, growth3, gap4new, growth4new";
"-----------------------------------------------------------";


//Legend
label = "";
location = "";
orientation = 0;
plotSetLegend(&myPlot, label, location, orientation);

plotLayout(2, 2, 3);
//Set the title, title font and title font size 
plotSetTitle(&myPlot, "Cumulative", "verdana", 18);
// compute weights
weights    = cumsumc(predlhall);
weights    = exp(weights - weights[.,1]);
weights    = weights./sumc(weights');
s1weighted = sumc( (weights.*s1all)' );
// Plot the graph
plotXY(myPlot,ti,s1weighted);

ti~s1weighted;
"Plot 2.1";
"Period versus weighted prob of s=1, full sample";
"-----------------------------------------------";


plotLayout(2, 2, 4);
//Set the title, title font and title font size 
plotSetTitle(&myPlot, "Cumulative - Post ZLB", "verdana", 18);
// compute weights
if fileext $== "us";
   Tfirst = 2009.00;
   Tlast  = 2015.00;
else;
   Tfirst = 1999.00;
   Tlast  = 2015.00;
endif;	
Tfirst_ind    = maxindc(ti .== Tfirst);
Tlast_ind     = maxindc(ti .== Tlast);
weights    = cumsumc(predlhall[Tfirst_ind:Tlast_ind,.]);
weights    = exp(weights - weights[.,1]);
weights    = weights./sumc(weights');
s1weighted = sumc( (weights.*s1all[Tfirst_ind:Tlast_ind,.])' );
// Plot the graph
plotXY(myPlot,ti[Tfirst_ind:Tlast_ind,.],s1weighted);

ti[Tfirst_ind:Tlast_ind,.]~s1weighted;
"Plot 2.2";
"Period versus weighted prob of s=1, full sample";
"-----------------------------------------------";

end;

dim = 20|15;
plotSave("PredLogLHAndS1" $+ fileext $+".pdf",dim);
