/********************************************************************
*  Global Settings
*
*  Last updated: 
********************************************************************/
dlibrary -a gensys; 
library pgraph, user;

// declare globals
declare matrix global, eta, tau, kappa, psi1, rhoR,
        rhog, rhod, rhoz, sigR, sigg, sigd, sigz, 
        nu, chi_h, gstar, psi2, gamQ, rA, piA, phi, pibar;

modelid = 51;

// select sample
// jp
// us
dataselstr = "us";

// load parameters
if  modelid == 51;
   lmodel = "sunspot14_095099";
   rho0 = 0.95;
   rho1 = 0.99;
   if dataselstr == "us";
     #include sunspot14_095099_us\input\par_us.txt;
   endif;
elseif modelid == 52;
   lmodel = "sunspot14_092099";
   rho0 = 0.92;
   rho1 = 0.99;
   if dataselstr == "jp";
     #include sunspot14_092099_jp\input\par_jp.txt;
   endif;
endif;

#include pathspec.g

if dataselstr $== "us";
   load path = ^inputpath theta[]= "theta_us.txt";   
   //load path = ^inputpath shocks_mean[]= "shocks_mean_us.txt";
elseif dataselstr $== "jp";
   load path = ^inputpath theta[]= "theta_jp.txt";   
endif;

//shocks_mean = reshape(shocks_mean,rows(shocks_mean)/3,3);
shocks_mean = zeros(1,3);

// set additional globals
ind_impose  = 1;
n_particles = 100;
n_particles_initial = 100;
n_states    = 30+3;
Rbar = 1;
// particles: 
// 1-5:   pis, Rs, ys, cs, hs 
// 6-11:  ws, ee1s, ers, egs, ezs, eds 
// 12-19: ds, zs, gs, CCs, YYs, GGs, TTs, As, 
// 20-23: SUN, delys, delcs, trends,
// 24-30: R_lag, y_lag, c_lag, d_lag, z_lag, g_lag, SUN_lag
n_var     = 3;
n_shocks  = 4;
hpdprob   = 0.9;
n_irf     = 12;
n_var_irf = 4;

// Choose proposal in filter
// 1 = shocks are centered at zero
// 2 = shocks are centered at mean value from linear KF
// 3 = shocks are read from external file
proposal_select_ = 0;

// solution of linear model
// model specification
//  MSEL[1]:
//   1 without measurement error
//   2 with measurement errors
//
//  MSEL[2]:
//   1 pibar = 1
//   2 pibar = pistar
msel = zeros(2,1);
msel[1] = 2;
msel[2] = 1;

// parameter vector
para = eta|tau|kappa|psi1|rhoR|
       rhog|rhod|rhoz|sigR|sigg|
	   sigd|sigz|nu|chi_h|gstar|
	   psi2|gamQ|rA|piA|phi|
	   pibar|alp|lnA0;

// eps_grid
eps_min0  = -2;
eps_max0  =  2;
eps_n0     = 5;
eps_grid0 = seqa(eps_min0,(eps_max0-eps_min0)/(eps_n0-1),eps_n0);

// eps_grid
eps_min1  = -0.5;
eps_max1  =  0.5;
eps_n1     = 11;
eps_grid1 = seqa(eps_min1,(eps_max1-eps_min1)/(eps_n1-1),eps_n1);

// eps_grid
eps_min01  = -2;
eps_max01  =  2;
eps_n01     = 9;
eps_grid01 = seqa(eps_min01,(eps_max01-eps_min01)/(eps_n01-1),eps_n01);
