/******************************************************************/
/* filename :       dsge_filter.g                                 */
/* Description:                                                   */
/*                                                                */
/******************************************************************/

new;
closeall; 
cls;

	
#include dsge_spec.g; 
theta_in  = theta;
ncoefs    = rows(theta)/8;


// _forcetilde = 10 means tilde regime starting in 2009Q1
// _forcetilde = 11 means star regime starting in 2009Q1
_forcetilde = 0;

if _forcetilde == 10;
   fileext = "_gs000";
elseif _forcetilde == 11;
   fileext = "_gs111";
else;
   fileext = "_gs";
endif;

// _changeME
_scaleME = 4;
if _scaleME == 1;
   // measurement errors
   sigmy = 0.2*sigmy;
   sigmc = 0.2*sigmc;	
   sigmp = 0.2*sigmp;
   sigmr = 0.2*sigmr;
   fileext = fileext $+ "ME02";
elseif _scaleME == 2;
   sigmy = sqrt(0.5)*sigmy;
   sigmc = sqrt(0.5)*sigmc;	
   sigmp = sqrt(0.5)*sigmp;
   sigmr = sqrt(0.5)*sigmr;
   fileext = fileext $+ "ME07";
elseif _scaleME == 3;
   // measurement errors
   sigmy = 0.3*sigmy;
   sigmc = 0.3*sigmc;	
   sigmp = 0.3*sigmp;
   sigmr = 0.3*sigmr;
   fileext = fileext $+ "ME03";
elseif _scaleME == 4;
   sigmy = sqrt(0.5)*sigmy;
   sigmc = sqrt(0.5)*sigmc;	
   sigmp = sqrt(0.5)*sigmp;
   sigmr = 0.2*sigmr;
   fileext = fileext $+ "ME07R02";
endif;

// if we use log-level of output:
sigmy = sigmy/100;

// _forcestart
if dataselstr $== "us";
   _forcestart = 2009;
   _forceend   = 2015;
   //_forcestart = 1985;
elseif dataselstr $== "jp";
   _forcestart = 1999;
   _forceend = 2015;
endif;


#include dsge_linearmod.src;
// compute steady states
{bet,r,gam,pibar,pi_ss,c_ss,y_ss,R_ss,ee1_ss,
 pi_tilde,c_tilde,y_tilde,R_tilde,ee1_tilde} = dsgestst(para,msel);

sigR    = para[9];
sigg    = para[10];
sigd    = para[11];
sigz    = para[12];

// begin insert
// solve linear model
{TT,QQ,RR,HH,DD,ZZ,VV,RC}=sysmat((para|sigmy|sigmc|sigmp|sigmr),msel);


// load data
#include dsge_loaddata.g;
n_periods  = rows(YY);
period_end = n_periods;
//period_end = 10;
shocks_sig = 1;
lambda     = 0.5;

// Compute initialization for Kalman Filter
// The states are
// 1-5 : R, y, z, g, pi
// 6-10: d, ytrend, c, e_R, e_g,
// 11-15: e_z, e_d, y_lag, c_lag, A

// use unconditional mean for stationary states and 0 for log technology
n_linstates = rows(TT);
At       = zeros(n_linstates,1);
Pt       = zeros(n_linstates,n_linstates);
Pt[1:n_linstates-1,1:n_linstates-1] 
         = dlyap(TT[1:n_linstates-1,1:n_linstates-1],RR[1:n_linstates-1,.]
		   *QQ*RR[1:n_linstates-1,.]');

// Update the states in view of the initial observation
// on the trend of output so that we can track the trend correctly
// we treat A0 as known
At[15]   = 0.5*(YY[1,6]-ln(y_ss)-lnA0)+0.5*(YY[1,5]-ln(y_ss)-lnA0);
Pt[.,15] = zeros(n_linstates,1);
Pt[15,.] = zeros(1,n_linstates);

// Conditional on A0, we can compute y and ytrend and treat these
// deviations as known.
// We then need the conditional distribution of the remaining states given
// y and ytrend
// Construct a matrix that re-orders the states
// order y (2) and ytrend (7) last; delete A (15)
MM    = eye(n_linstates);
MM    = MM[1 3 4 5 6 8 9 10 11 12 13 14 2 7,.];

// compute mean and covariance of block 1 given block 2 (y and ytrend)
PtMM   = MM*Pt*MM';
PtMM11 = PtMM[1:12,1:12];
PtMM21 = PtMM[13:14,1:12];
PtMM12 = PtMM[1:12,13:14];
PtMM22 = PtMM[13:14,13:14];
PtMM1c2 = PtMM11 - PtMM12*inv(PtMM22)*PtMM21;

AtMM1c2 = PtMM12*inv(PtMM22)*( YY[1,5:6]' -ln(y_ss)-lnA0 - At[15]);


// re-order the mean and covariance matrix, after treating y, ytrend, and A as known
PtMM            = zeros(n_linstates,n_linstates);
PtMM[1:12,1:12] = PtMM1c2;
AtMM            = AtMM1c2 |
                  (YY[1,5]' -ln(y_ss)-lnA0 - At[15]) |
				  (YY[1,6]' -ln(y_ss)-lnA0 - At[15]) |
				  At[15];

MMinv = eye(n_linstates);
MMinv = MMinv[1 13 2 3 4 5 14 6 7 8 9 10 11 12 15,.];

At = MMinv*AtMM;
Pt = MMinv*PtMM*MMinv';

// Run Kalman filter for 3 steps
// Initializations

i_period  = 2;
do until i_period > 4; 

   // Kalman filtering
   // *************************************************************	
   // data: GDP Growth, Consumption Growth, Infl, FFR, log Y, log Trend, log CBO, log C
   // model: logY, Infl, R

   // selection matrix
   if YYmask[i_period] == 0;
	  MM = MM0;
   elseif YYmask[i_period] == 1;
	  MM = MM1;
   endif; 
	
   // Forecasting and Likelihood Evaluation	
   alphahat      = TT*At; 
   yy_hat_kf     = (DD[1 3 4] + ZZ[1 3 4,.]*alphahat + (ln(gam) | 0 | 0)*(i_period-1) )'*MM;
   Phat          = TT*Pt*TT' + RR*QQ*RR';
   Phat          = .5*(Phat+Phat');
   Ft            = ZZ[1 3 4,.]*Phat*ZZ[1 3 4,.]' + HH[1 3 4,1 3 4];
   Ft            = .5*(Ft+Ft');
   Ft            = MM'*Ft*MM;
   yy_t          = YY[i_period,5 3 4 ]*MM;                	
   yy_e_kf       = yy_t  - yy_hat_kf;
						
   // Kalman filter updating for linear model
   At   = alphahat + Phat*ZZ[1 3 4,.]'*MM*inv(Ft)*yy_e_kf';
   Pt   = Phat - (Phat*ZZ[1 3 4,.]'*MM+RR*VV[.,1 3 4]*MM)*inv(Ft)*(Phat*ZZ[1 3 4,.]'*MM+RR*VV[.,1 3 4]*MM)';

   i_period = i_period+1;
endo;

// Extract relevant submatrix from At and Pt
// The states are
// 1-5 : R, y, z, g, pi
// 6-10: d, ytrend, c, e_R, e_g,
// 11-15: e_z, e_d, y_lag, c_lag, A
//
// We need init: R, y, c, d, z, g, A, trend
Atsub = At[1 2 8 6 3 4 15];
Ptsub = Pt[1 2 8 6 3 4 15,1 2 8 6 3 4 15];

//rndseed 123;
Atsim = Atsub + cholpsd(Ptsub)*rndn(7,2*n_particles);

// Conversion
init_sim = R_ss*exp(Atsim[1,.]) |
           y_ss*exp(Atsim[2,.]) |
		   c_ss*exp(Atsim[3,.]) |
		   Atsim[4,.]|
		   Atsim[5,.]|
		   ln(gstar)+Atsim[6,.] |
		   exp(Atsim[7,.]);
init_sim = init_sim';

// Check to make sure bounds are satisfied
init_boundsviolated =   (init_sim[.,1] .< R_min)
                    .OR (init_sim[.,1] .> R_max)
					.OR (init_sim[.,4] .< d_min)
					.OR (init_sim[.,4] .> d_max)
					.OR (init_sim[.,5] .< z_min)
					.OR (init_sim[.,5] .> z_max)
					.OR (init_sim[.,6] .< g_min)
					.OR (init_sim[.,6] .> g_max);



init_sim = delif(init_sim,init_boundsviolated);
if rows(init_sim) < n_particles;
   "not enough initial particles"; end;
else;
   init_sim = init_sim[1:n_particles,.];	
endif;

meanc(init_sim)';


// Initializations of output matrices
particle_weights_old = ones(n_particles,1)/n_particles;
particle_values_old  = zeros(n_particles,n_states);

particle_weights_new = ones(n_particles,1)/n_particles;
particle_values_new  = zeros(n_particles,n_states);

ess   = zeros(n_periods,1);
loglh = zeros(n_periods,1);


// Load initial values for particles
//lparticles = resupath $+ "\\initialparticles.dat";
//open fhparticles = ^lparticles for read;
//particle_values_old = readr(fhparticles,n_particles);
//closeall;

// Create Output files and write initial particles
oparticle_values = resupath $+ "\\particle_values" $+ fileext $+ ".dat";
create fhparticle_values = ^oparticle_values with STATE, (1+n_particles*n_states), 4;

oparticle_weights = resupath $+ "\\particle_weights" $+ fileext $+ ".dat";
create fhparticle_weights = ^oparticle_weights with Weights, (1+n_particles), 4;

oparticle_loglh   = resupath $+ "\\particle_loglh" $+ fileext $+ ".dat";
create fhparticle_loglh = ^oparticle_loglh with LogLH, (1+2), 4;


// Start Log file
outstr = resupath  $+ "\\" $+ ftos(timedt,"%*.*lf",1,0) $+ "filter" $+ fileext $+ ".log"; 
output file = ^outstr reset;

"Model             " modelid $lmodel;
"Data              " $dataselstr;
"Initial Particles " n_particles_initial;
"Particles         " n_particles;
"Measurement errors y, c, p, r" sigmy sigmc sigmp sigmr ;
"--------------------------------------------";
"";
pause(1e-5);


// Begin filtering

var_measure     = eye(n_var).*(sigmy^2|sigmp^2|sigmr^2);
var_measure_inv = inv(var_measure);

i_period  = 5;
do until i_period > period_end; //n_periods;

   "Processing Period" ti[i_period];
   // set start time
   tstart = date;

   if i_period > 5;
      "States from period" ti[i_period-1];
	  // compute average state
      // init: R, y, c, d, z, g, A 	
      // 1-5:   pis, Rs, ys, cs, hs 
      // 6-11:  ws, ee1s, ers, egs, ezs, eds 
      // 12-18: ds, zs, gs, CCs, YYs, GGs, As, 
      // 19-21: SUN, delys, delcs, 
      // 22-28: R_lag, y_lag, c_lag, d_lag, z_lag, g_lag, SUN_lag 	 
      init_mean = meanc(particle_values_old[.,2 3 4 12 13 14 18])';
	   
	  // Construct states for linear model from particles
	  particle_values_old_mean = meanc(particle_values_old);
	  // Composition of Atlag
      // 1-5 : R, y, z, g, pi
      // 6-10: d, ytrend, c, e_R, e_g,
      // 11-15: e_z, e_d, y_lag, c_lag, A
      Atlag = zeros(15,1);
	   
	  Atlag[1] = ln(particle_values_old_mean[2]/R_ss);
	  Atlag[2] = ln(particle_values_old_mean[3]/y_ss);
	  Atlag[3] = particle_values_old_mean[13];
	  Atlag[4] = particle_values_old_mean[14]-ln(gstar); 
	  Atlag[5] = ln(particle_values_old_mean[1]/pi_ss);
	  
	  Atlag[6] = particle_values_old_mean[12];
	  Atlag[7] = ln(particle_values_old_mean[3]/y_ss); //not used
	  Atlag[8] = ln(particle_values_old_mean[4]/c_ss); 
	  Atlag[9] = particle_values_old_mean[8]/sigR; 
	  Atlag[10]= particle_values_old_mean[9]/sigg;
	  
	  Atlag[11]= particle_values_old_mean[10]/sigz;
	  Atlag[12]= particle_values_old_mean[11]/sigd;
	  Atlag[13]= ln(particle_values_old_mean[23]/y_ss);
	  Atlag[14]= ln(particle_values_old_mean[24]/c_ss);
	  Atlag[15]= ln(particle_values_old_mean[18]);
	   
   elseif i_period == 5;
	  init_mean = meanc(init_sim)';
	  Atlag = At; // from Kalman filter iterations above 
   endif;
  
   // selection matrix
   if YYmask[i_period] == 0;
	  MM = MM0;
   elseif YYmask[i_period] == 1;
	  MM = MM1;
   endif; 
 
	
   // One iteration of KF
   alphahat      = TT*Atlag; 
   yy_hat_kf     = (DD[1 3 4] + ZZ[1 3 4,.]*alphahat + (ln(gam) | 0 | 0) )'*MM;
   Phat          = RR*QQ*RR';
   Phat          = .5*(Phat+Phat');
   Ft            = ZZ[1 3 4,.]*Phat*ZZ[1 3 4,.]' + HH[1 3 4,1 3 4];
   Ft            = MM'*Ft*MM;
   Ft            = .5*(Ft+Ft');
   yy_t          = YY[i_period,5 3 4 ]*MM;
   yy_e_kf       = yy_t  - yy_hat_kf;
   
   "KF Prediction";
   "pred  " yy_hat_kf;
   "actual" yy_t;
   

   // Kalman filter updating for linear model
   At   = alphahat + Phat*ZZ[1 3 4,.]'*MM*inv(Ft)*yy_e_kf';

   shocks_mean1_kf = At[9]  ~
                     At[10] ~
                     At[11] ~
                     At[12] ;
   yy_hat_kf2     = (DD[1 3 4] + ZZ[1 3 4,.]*alphahat + (ln(gam) | 0 | 0) 
                   + ZZ[1 3 4,.]*RR*shocks_mean1_kf' )'*MM;
   
   "recon " yy_hat_kf2;
   
   eps_grid_in = eps_grid0~eps_grid0~eps_grid0~eps_grid0;

   sun_in = 0;
   {shocks_mean0,fcn_minimum0} = gridsearch(eps_grid_in,init_mean);

   sun_in = 1;
   {shocks_mean1,fcn_minimum1} = gridsearch(eps_grid_in,init_mean); 

   
   "Mean of Shocks:";
   "Deflation Regime" shocks_mean0;
   "e'e, min        " (shocks_mean0*shocks_mean0')~fcn_minimum0;
   "Targ Infl Regime" shocks_mean1; 
   "Kalman Filter   " shocks_mean1_kf; 
   "e'e, min        " (shocks_mean1*shocks_mean1')~fcn_minimum1;
   pause(1e-5);

   shocks_weight = zeros(n_particles,1);

   i_particle = 1;
   do until i_particle > n_particles;
      
         //propagate particles one-period ahead

         // init_ss: R_lag, y_lag, z_lag, g_lag, A_lag, sun
	     // 1-5:   pis, Rs, ys, cs, hs 
         // 6-11:  ws, ee1s, ers, egs, ezs, eds 
         // 12-18: ds, zs, gs, CCs, YYs, GGs, As, 
         // 19-21: SUN, delys, delcs, 
         // 22-28: R_lag, y_lag, c_lag, d_lag, z_lag, g_lag, SUN_lag
	     if i_period > 5;
            init_all = particle_values_old[i_particle,.];
            init_ss  = init_all[1,2 3 4 12 13 14 18];
            init_sun = init_all[1,19];
	     elseif i_period == 5;
            init_ss  = init_sim[i_particle,.];
            init_sun = 1;			  
	     endif;
	   	   
         // draw sunspot shock 
	     // Option 1: forward simulation
         sun_in    = get_sunspot(1,init_sun,0,1);
	     pprob = init_sun*rho1+(1-init_sun)*(1-rho0);
	     qprob = init_sun*rho1+(1-init_sun)*(1-rho0);
		 //goto eofoption2;
 
	     // Option 2: sun=1 with prob lambda
         if rndu(1,1) > lambda;
            // generate draws from star regime
            sun_in = 1;
            // compute prior
            // if s(t-1)=1, then s(t)=1 with prob rho1
            // if s(t-1)=0, then s(t)=1 with prob (1-rho0)
            pprob = init_sun*rho1+(1-init_sun)*(1-rho0);
            qprob = 1-lambda;
         else;
            // generate draws from tilde regime
            sun_in = 0;
            // compute prior
            // if s(t-1)=1, then s(t)=0 with prob (1-rho1)
            // if s(t-1)=0, then s(t)=0 with prob rho0
            pprob = init_sun*(1-rho1)+(1-init_sun)*rho0;
            qprob = lambda;
         endif;
		 
		 eofoption2:

         if (_forcetilde == 10) 
			 AND (ti[i_period] GE _forcestart)
			 AND (ti[i_period] LE _forceend);
			sun_in = 0;
			pprob = 1;
			qprob = 1;
         elseif (_forcetilde == 11) 
			 AND (ti[i_period] GE _forcestart)
			 AND (ti[i_period] LE _forceend);
			sun_in = 1;
			pprob = 1;
			qprob = 1;
		  endif;
		 
         if i_period > 0;
            if sun_in == 0;       
               shocks_mean = shocks_mean0;
            else;
               shocks_mean = shocks_mean1;
            endif;
         else;
            shocks_mean = zeros(1,n_shocks);
         endif; 
         
		 
         shocks_in = shocks_mean + shocks_sig*rndn(1,n_shocks);
         shocks_weight[i_particle,.] 
                   = (pprob/qprob)
		           * (shocks_sig^n_shocks)*exp( -0.5*shocks_in*shocks_in' 
                   + 0.5*(shocks_in-shocks_mean)*(shocks_in-shocks_mean)'/shocks_sig^2 );

         // outsim:
         // 1-5:   pis, Rs, ys, cs, hs 
         // 6-11:  ws, ee1s, ers, egs, ezs, eds 
         // 12-18: ds, zs, gs, CCs, YYs, GGs, As, 
         // 19-21: SUN, delys, delcs, 
         // 22-28: R_lag, y_lag, c_lag, d_lag, z_lag, g_lag, SUN_lag
         // particles: outsim  
         //            my mp mR
         
         {outsim,valid} = simulate_sunspot(theta_in,shocks_in,init_ss,sun_in,0);

         // update states
         p_v = real(outsim[1,.]);

         // prediction of output growth, inflation, and Fed Funds rate		 
		 yy_hat = ln(outsim[1,16])  ~ 
                  400*ln(outsim[1,1]) ~ 
                  400*ln(outsim[1,2]); 
	     yy_hat = yy_hat*MM;
	     yy_t   = YY[i_period,5 3 4 ]*MM;
         yy_e   = yy_t-yy_hat;
		 if valid == 0;
			yy_e = ones(1,cols(yy_e))*1E8;
		 endif;

         // augment p_v by measurement errors
         p_v = p_v ~ yy_e*MM';
         particle_values_new[i_particle,.] = p_v;
  
         // compute raw weight of particle
         p_w = (2*pi)^(-cols(yy_t)/2)*det(MM'*var_measure_inv*MM)^(0.5)
               *exp(-0.5*yy_e*(MM'*var_measure_inv*MM)*yy_e')*particle_weights_old[i_particle,1]*shocks_weight[i_particle,.] ; 
         particle_weights_new[i_particle,1] = p_w;  

         // Screen output
         if fmod(i_particle,n_particles/10)==0; ".";; pause(1e-20); endif;

         i_particle = i_particle + 1;
      
   endo;
   "";
 
   // compute likelihood
   loglh[i_period] = real( ln( sumc(particle_weights_new)/sumc(particle_weights_old) ) );
   "log likelihood Contribution  " loglh[i_period];
   
   // compute normalized weights
   p_w = particle_weights_new; 
   p_w = p_w/maxc(p_w);
   p_w = real(p_w/sumc(p_w));
  
   // compute effective sample size
   ess[i_period] = 1/sumc( p_w^2 );
   "Effective Number of Particles" ess[i_period];

   // potentially re-sample particles
   if ess[i_period] < n_particles/3;
      "Resampling";
      outindex = deterministicR(seqa(1,1,n_particles),p_w);
      particle_values_new = particle_values_new[outindex,.];
      p_w = ones(n_particles,1)/n_particles;
   endif;

   particle_weights_new = p_w;
   
   "Probability of target infl regime" meanc(particle_values_new[.,19]); 
   
   //set finish time and print elapsed time
   tfin = date;
   "Elapsed Time"; etstr(ethsec(tstart,tfin));
   "--------------------------------------------";
   "";
   pause(1e-5);


   wr = writer(fhparticle_values,(ti[i_period]~real( vec( particle_values_new')' ) ));
   wr = writer(fhparticle_weights,(ti[i_period]~real(particle_weights_new')));

   particle_values_old  = particle_values_new;
   particle_weights_old = particle_weights_new;

   i_period = i_period+1;
endo;
      

"Period  Loglh(t)  ESS";
ti~loglh~ess;
"";

"Loglh";
sumc(loglh);
"";

wr = writer(fhparticle_loglh,(ti[2:period_end,.]~loglh[2:period_end,.]~ess[2:period_end,.]));


"ESS Summary Statistics";
ess = ess[2:period_end];
"mean" meanc(ess);
"stdd" stdc(ess);
"min " minc(ess);
"max " maxc(ess);

// close log file
output off;

end;

#include simulate_sunspot.src;
#include basis_temp.src;
#include deterministicR.src;
#include get_sunspot.src;


//====================================================

proc (2) = gridsearch(eps_grid,paststate);
local fcn_min, eps_min, fcn_eps,
      r_ind, g_ind, z_ind, d_ind, eps_r, eps_g, eps_z, eps_d,
      shocks, outsim, zz_hat, zz_t, zz_e, eps_n;

fcn_min = 1E8;
eps_min = zeros(1,4);
eps_n = rows(eps_grid);

r_ind = 1;
do until r_ind > eps_n;
   eps_r = eps_grid[r_ind,1];
   g_ind = 1;
   do until g_ind > eps_n;
      eps_g = eps_grid[g_ind,2];
      z_ind = 1;
      do until z_ind > eps_n;
         eps_z = eps_grid[z_ind,3];
		 d_ind = 1;
         do until d_ind > eps_n;
            eps_d = eps_grid[d_ind,4];
 
            shocks = eps_r ~ eps_g ~ eps_z ~ eps_d;

            {outsim,valid} = simulate_sunspot(theta_in,shocks,paststate,sun_in,0);
		  
            // prediction of output growth, inflation, and Fed Funds rate
            zz_hat = ln(outsim[1,16])  ~ 
                     400*ln(outsim[1,1]) ~ 
                     400*ln(outsim[1,2]);  
			zz_hat = zz_hat*MM; 
	        zz_t   = YY[i_period,5 3 4 ]*MM;
            zz_e   = zz_t-zz_hat;
			
			if valid == 0;
			   zz_e = ones(1,cols(zz_e))*1E8;
		    endif;
			 
			//"actual   " zz_t;
			//"predicted" zz_hat;
			//"************"; 
			 			 
            //fcn_eps = zz_e*zz_e';
            fcn_eps = 0.5*zz_e*(MM'*var_measure_inv*MM)*zz_e' +0.5*shocks*shocks';

            if fcn_eps < fcn_min;
               fcn_min = fcn_eps;
               eps_min = shocks;
            endif;
			
			d_ind = d_ind+1;
         endo;
         z_ind = z_ind+1;
      endo;
      g_ind = g_ind + 1;
   endo;
   r_ind = r_ind + 1;
endo; 

retp(eps_min,fcn_min);
endp;
