/*************************************************************
** filename:    dsge_statemom.g
** description: Compute  mean and based on
**              output of particle filter
*************************************************************/

new;
closeall; 
cls;

#include dsge_spec.g; 

// compute steady states
{bet,r,gam,pibar,pi_ss,c_ss,y_ss,R_ss,ee1_ss,
 pi_tilde,c_tilde,y_tilde,R_tilde,ee1_tilde} = dsgestst(para,msel);

// Load Particles
// particle_values
// each row: ti, n_particles*n_states

//fileext = "_gs111ME02";
//fileext = "_gs000";
fileext = "_gsME07R02";

lparticles       = resupath $+ "\\particle_values" $+ fileext  ;
open fhparticles = ^lparticles for read;

ti = 0;
state_mean = zeros(1,n_states);
state_low  = zeros(1,n_states);
state_high = zeros(1,n_states);


// Start Log file
outstr = resupath  $+ "\\" $+ ftos(timedt,"%*.*lf",1,0) $+ "statemom" $+ fileext $+ ".log"; 
output file = ^outstr reset;

"Model             " modelid $lmodel;
"Data              " $dataselstr;
"Initial Particles " n_particles_initial;
"Particles         " n_particles;
"Measurement errors y, p, r" sigmy sigmp sigmr;
"--------------------------------------------";
"";
pause(1e-5);


eofloop = 0;

do until eofloop; 

   // read particles for a particular time period
   particles = readr( fhparticles, 1 );
   
   // re-arrange particles
   ti = ti| particles[1];
   "Processing Period" particles[1];
   pause(1E-5);
   particles = reshape(particles[2:n_particles*n_states+1],n_particles,n_states); 

 
   // Compute mean and HPD interval
   state_ti_mean = meanc(particles)';
   state_ti_ci   = zeros(2,n_states);
   j = 1;
   do until j > n_states;
      state_ti_ci[.,j] = hpdint(particles[.,j],hpdprob);
      j = j+1;
   endo;

   state_mean = state_mean | state_ti_mean;
   state_low  = state_low  | state_ti_ci[1,.];
   state_high = state_high | state_ti_ci[2,.];

   eofloop = eof(fhparticles);

endo;

closeall;

// remove first row of zeros
n_ti = rows(state_mean)-1;
ti = ti[2:n_ti+1,.];
state_mean = state_mean[2:n_ti+1,.];
state_low  = state_low[2:n_ti+1,.];
state_high = state_high[2:n_ti+1,.];


//All States;
"All States";
ti~state_mean;
"";


// Save Results
ostates = resupath $+ "\\states" $+ fileext ;
create fhstates = ^ostates with STATES, 3*n_states+1, 8;
wr = writer(fhstates, ti~state_low~state_mean~state_high );
closeall; 

// close log
output off;

end;

//#include simulate_sunspot.src;
//#include basis_temp.src;
#include hpdint.src;
#include dsge_linearmod.src;

