/******************************************************************/
/* filename :       dsge_simulate.g                               */
/* Description:                                                   */
/*                                                                */
/******************************************************************/

new;
closeall; 
cls;

#include dsge_spec.g; 
#include dsge_linearmod.src;

//simulate data from linearized and nonlinear DSGE model

rndseed 123;

T         = 20;
Tdrop     = 0;
// shocks: er, eg, ez, ed
shocks_in = rndn(T+Tdrop,4); 
//shocks_in[.,1:3] = zeros(T+Tdrop,3); 
theta_in  = theta;
ncoefs    = rows(theta)/8;

// compute steady states
{bet,r,gam,pibar,pi_ss,c_ss,y_ss,R_ss,ee1_ss,
 pi_tilde,c_tilde,y_tilde,R_tilde,ee1_tilde} = dsgestst(para,msel);
 
// initialize simulation
// init: R, ylag, c, d, z, g, A, trend
init_ss  = R_ss|y_ss|c_ss|0|0|ln(gstar)|1|y_ss;

// simulate nonlinear model
sim_length    = T+Tdrop;
sun_sim       = ones(sim_length,1);
outsim_nonlin = simulate_sunspot(theta_in,shocks_in,init_ss,sun_sim,Tdrop);

// convert into observables
// output, output trend, output growth, consumption growth, inflation, interest rates
// outsim:
// 1-5:   pis, Rs, ys, cs, hs 
// 6-11:  ws, ee1s, ers, egs, ezs, eds 
// 12-19: ds, zs, gs, CCs, YYs, GGs, TTs, As, 
// 20-23: SUN, delys, delcs, trends,
// 24-30: R_lag, y_lag, c_lag, d_lag, z_lag, g_lag, SUN_lag


YY_sim_nonlinear = ln(outsim_nonlin[.,16]) ~
                   ln(outsim_nonlin[.,18]) ~
		           100*(ln(gam)+ln(outsim_nonlin[.,3])-ln(outsim_nonlin[.,25])+outsim_nonlin[.,13]) ~
		           100*(ln(gam)+ln(outsim_nonlin[.,4])-ln(outsim_nonlin[.,26])+outsim_nonlin[.,13]) ~
                   400*ln(outsim_nonlin[.,1]) ~ 
                   400*ln(outsim_nonlin[.,2]); 

// Simulate linearized DSGE model 

// solve linear model
{TT,QQ,RR,HH,DD,ZZ,VV,RC}=sysmat((para|sigmy|sigmc|sigmp|sigmr),msel);

// The states (in deviations from steady state) are
// 1-5 : R, y, z, g, pi
// 6-10: d, ytrend, c, e_R, e_g,
// 11-15: e_z, e_d, y_lag, c_lag, A
n_states = rows(TT);
At       = zeros(n_states,1);

YY_sim_linear = zeros(T+Tdrop,6);
tind = 1;
do until tind > (T+Tdrop);
   At = TT*At + RR*shocks_in[tind,.]';
   At';
   YY_sim_linear[tind,1:5] 
	  =  (DD + ZZ*At + (ln(gam) | 0 | 0 | 0|ln(gam))*tind )';
   YY_sim_linear[tind, 6] 
	  =  100*(ln(gam) + At[2] + At[3] - At[13]);
   tind = tind+1;
endo;
YY_sim_linear = YY_sim_linear[Tdrop+1:Tdrop+T, 1 5 6 2 3 4];

// Comparison

"Log Output: Nonlinear vs. Linear";
YY_sim_nonlinear[.,1]~YY_sim_linear[.,1];

"Log Output Trend: Nonlinear vs. Linear";
YY_sim_nonlinear[.,2]~YY_sim_linear[.,2];

"Log Output Gap: Nonlinear vs. Linear";
100*(YY_sim_nonlinear[.,1]-YY_sim_nonlinear[.,2])~100*(YY_sim_linear[.,1]-YY_sim_linear[.,2]);

"Output Growth: Nonlinear vs. Linear";
YY_sim_nonlinear[.,3]~YY_sim_linear[.,3];

"Consumption Growth: Nonlinear vs. Linear";
YY_sim_nonlinear[.,4]~YY_sim_linear[.,4];

"Inflation: Nonlinear vs. Linear";
YY_sim_nonlinear[.,5]~YY_sim_linear[.,5];

"Interest: Nonlinear vs. Linear";
YY_sim_nonlinear[.,6]~YY_sim_linear[.,6];




 	
	



end;

#include simulate_sunspot.src;
#include basis_temp.src;
#include get_sunspot.src;
#include dsge_linearmod.src;
